import java.util.ArrayList;
import java.util.List;

public class RoleInfo {
    private String name; // 角色名称
    private int health; // 生命值
    private int attackPower; // 攻击力
    private double headshotProbability; // 爆头概率
    private List<Buff> buffs; // Buff列表
    private List<Skill> skills; // 技能列表

    // 构造函数
    public RoleInfo(String name, int health, int attackPower, double headshotProbability) {
        this.name = name;
        this.health = health;
        this.attackPower = attackPower;
        this.headshotProbability = headshotProbability;
        this.buffs = new ArrayList<>();
        this.skills = new ArrayList<>();
    }

    // 获取和设置角色名称
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    // 获取和设置生命值
    public int getHealth() {
        return health;
    }

    public void setHealth(int health) {
        this.health = health;
    }

    // 获取和设置攻击力
    public int getAttackPower() {
        return attackPower;
    }

    public void setAttackPower(int attackPower) {
        this.attackPower = attackPower;
    }

    // 获取和设置爆头概率
    public double getHeadshotProbability() {
        return headshotProbability;
    }

    public void setHeadshotProbability(double headshotProbability) {
        this.headshotProbability = headshotProbability;
    }

    // 添加Buff
    public void addBuff(Buff buff) {
        buffs.add(buff);
        buff.apply(this);
    }

    // 移除Buff
    public void removeBuff(Buff buff) {
        buffs.remove(buff);
        buff.remove(this);
    }

    // 添加技能
    public void addSkill(Skill skill) {
        skills.add(skill);
    }

    // 使用技能
    public void useSkill(String skillName) {
        for (Skill skill : skills) {
            if (skill.getName().equals(skillName)) {
                skill.activate(this);
                break;
            }
        }
    }

    // 更新角色状态（每帧调用）
    public void update() {
        buffs.removeIf(buff -> !buff.update(this)); // 更新并移除过期的Buff
    }

    @Override
    public String toString() {
        return "RoleInfo{" +
                "name='" + name + '\'' +
                ", health=" + health +
                ", attackPower=" + attackPower +
                ", headshotProbability=" + headshotProbability +
                ", buffs=" + buffs +
                ", skills=" + skills +
                '}';
    }

    // Buff类
    public static class Buff {
        private String name;
        private int duration; // 持续时间（帧数）
        private int effectValue; // 效果值

        public Buff(String name, int duration, int effectValue) {
            this.name = name;
            this.duration = duration;
            this.effectValue = effectValue;
        }

        public void apply(RoleInfo role) {
            // 应用Buff效果
            role.setAttackPower(role.getAttackPower() + effectValue);
        }

        public void remove(RoleInfo role) {
            // 移除Buff效果
            role.setAttackPower(role.getAttackPower() - effectValue);
        }

        public boolean update(RoleInfo role) {
            // 更新Buff状态
            if (duration > 0) {
                duration--;
                return true;
            } else {
                remove(role);
                return false;
            }
        }

        @Override
        public String toString() {
            return "Buff{" +
                    "name='" + name + '\'' +
                    ", duration=" + duration +
                    ", effectValue=" + effectValue +
                    '}';
        }
    }

    // Skill类
    public static class Skill {
        private String name;
        private int damage;

        public Skill(String name, int damage) {
            this.name = name;
            this.damage = damage;
        }

        public String getName() {
            return name;
        }

        public void activate(RoleInfo role) {
            // 激活技能
            System.out.println(role.getName() + " 使用了技能 " + name + "，造成 " + damage + " 点伤害！");
        }

        @Override
        public String toString() {
            return "Skill{" +
                    "name='" + name + '\'' +
                    ", damage=" + damage +
                    '}';
        }
    }

    public static void main(String[] args) {
        // 创建角色
        RoleInfo player = new RoleInfo("Player1", 100, 50, 0.2);

        // 添加Buff
        player.addBuff(new Buff("攻击力增强", 100, 20));

        // 添加技能
        player.addSkill(new Skill("强力射击", 80));

        // 使用技能
        player.useSkill("强力射击");

        // 更新角色状态
        player.update();

        // 打印角色信息
        System.out.println(player);
    }
}