import java.awt.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

class Point {
    double x;
    double y;

    public Point(double x, double y) {
        this.x = x;
        this.y = y;
    }

    public double distance(Point other) {
        return Math.sqrt(Math.pow(other.x - x, 2) + Math.pow(other.y - y, 2));
    }

    public Point directionTo(Point other) {
        double dx = other.x - x;
        double dy = other.y - y;
        double length = Math.sqrt(dx * dx + dy * dy);
        return new Point(dx / length, dy / length);
    }
}

class Entity {
    Point position;
    double speed;

    public Entity(double x, double y, double speed) {
        this.position = new Point(x, y);
        this.speed = speed;
    }

    public void move(Point direction, int width, int height) {
        position.x += direction.x * speed;
        position.y += direction.y * speed;

        // Boundary check
        position.x = Math.max(0, Math.min(position.x, width));
        position.y = Math.max(0, Math.min(position.y, height));
    }
}

class Enemy extends Entity {
    private static final Random random = new Random();

    public Enemy(double x, double y, double speed) {
        super(x, y, speed);
    }

    public void randomMove(int width, int height) {
        double randomX = random.nextDouble() * 2 - 1;
        double randomY = random.nextDouble() * 2 - 1;
        Point direction = new Point(randomX, randomY).directionTo(new Point(0, 0));
        super.move(direction, width, height);
    }
}

class Player extends Entity {
    public Player(double x, double y, double speed) {
        super(x, y, speed);
    }

    public void chase(Entity target, int width, int height) {
        Point direction = position.directionTo(target.position);
        super.move(direction, width, height);
    }
}

class Obstacle {
    Point position;
    double radius;

    public Obstacle(double x, double y, double radius) {
        this.position = new Point(x, y);
        this.radius = radius;
    }

    public boolean collidesWith(Entity entity) {
        return position.distance(entity.position) <= radius;
    }
}

public class Game {
    private static final int WIDTH = 800;
    private static final int HEIGHT = 600;
    private static final double CAPTURE_RADIUS = 10.0;
    private static final int NUM_ENEMIES = 5;
    private static final int NUM_OBSTACLES = 3;

    public static void main(String[] args) {
        List<Enemy> enemies = new ArrayList<>();
        List<Obstacle> obstacles = new ArrayList<>();
        Random random = new Random();

        // Create enemies
        for (int i = 0; i < NUM_ENEMIES; i++) {
            double x = random.nextDouble() * WIDTH;
            double y = random.nextDouble() * HEIGHT;
            double speed = random.nextDouble() * 2 + 1; // Speed between 1 and 3
            enemies.add(new Enemy(x, y, speed));
        }

        // Create obstacles
        for (int i = 0; i < NUM_OBSTACLES; i++) {
            double x = random.nextDouble() * WIDTH;
            double y = random.nextDouble() * HEIGHT;
            double radius = random.nextDouble() * 50 + 20; // Radius between 20 and 70
            obstacles.add(new Obstacle(x, y, radius));
        }

        Player player = new Player(WIDTH / 4, HEIGHT / 4, 2);

        for (int i = 0; i < 100; i++) {
            // Move enemies
            for (Enemy enemy : enemies) {
                enemy.randomMove(WIDTH, HEIGHT);
            }

            // Move player to chase the closest enemy
            Enemy closestEnemy = enemies.stream()
                    .min((e1, e2) -> Double.compare(player.position.distance(e1.position), player.position.distance(e2.position)))
                    .orElse(null);

            if (closestEnemy != null) {
                player.chase(closestEnemy, WIDTH, HEIGHT);
            }

            // Check for collisions with obstacles
            for (Obstacle obstacle : obstacles) {
                if (obstacle.collidesWith(player)) {
                    System.out.println("Player collided with an obstacle!");
                    break;
                }
            }

            // Check if player captured any enemy
            for (Enemy enemy : enemies) {
                if (player.position.distance(enemy.position) <= CAPTURE_RADIUS) {
                    System.out.println("Player has captured the enemy at: (" + enemy.position.x + ", " + enemy.position.y + ")");
                    enemies.remove(enemy);
                    break;
                }
            }

            // Print positions
            System.out.println("Player at: (" + player.position.x + ", " + player.position.y + ")");
            for (Enemy enemy : enemies) {
                System.out.println("Enemy at: (" + enemy.position.x + ", " + enemy.position.y + ")");
            }

            // Exit if no more enemies
            if (enemies.isEmpty()) {
                System.out.println("All enemies captured!");
                break;
            }
        }
    }
}